==> GotoBLAS-1.12.tar.gz: GotoBLAS/lapack/getrf/getrf_single.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include "common.h"

#define GEMM_PQ  MAX(GEMM_P, GEMM_Q)

static FLOAT dm1 = -1.;

blasint CNAME(long m, long  n, FLOAT *a, long lda, blasint *ipiv, long offset, FLOAT *buffer){

  long j, js, jmin, is, imin, jc, jcmin;
  blasint iinfo, info;
  long jb, min_mn, blocking;
  FLOAT *offsetA, *offsetB, *offsetC;

  FLOAT *sa1, *sa2, *sb;
  sa1 = (FLOAT *)((long)buffer + GEMM_BUFFER_A_OFFSET);
  sa2 = (FLOAT *)((((long)sa1
		    + GEMM_PQ * GEMM_Q * COMPSIZE * SIZE + GEMM_ALIGN) & ~GEMM_ALIGN)
		  + GEMM_BUFFER_A_OFFSET);
  sb  = (FLOAT *)((((long)sa2
		    + GEMM_P  * GEMM_Q * COMPSIZE * SIZE + GEMM_ALIGN) & ~GEMM_ALIGN)
		  + GEMM_BUFFER_B_OFFSET);

  if (m == 0 || n == 0) return 0;
  
  if (n <= GEMM_UNROLL_N * 2) {
    info = GETF2(m, n, a, lda, ipiv, offset, buffer);
    return info;
  }
  
  info = 0;

  blocking = GEMM_Q;
  if (n <= 4 * GEMM_Q) blocking = n / 4;
  
  min_mn = MIN(m, n);

  for (j = 0; j < min_mn; j += blocking) {
    
    jb = min_mn - j;
    if (jb > blocking) jb = blocking;

    offsetA = a +  j       * lda * COMPSIZE;
    offsetB = a + (j + jb) * lda * COMPSIZE;

    iinfo = CNAME(m - j, jb, offsetA + j * COMPSIZE, lda, ipiv, j + offset, buffer);

    if (iinfo && !info) info = iinfo + j;

    if (j + jb < n) {
      
      TRSM_ILTCOPY(jb, jb, offsetA + j * COMPSIZE, lda, 0, sa1);
      
      for (js = 0; js < n - j - jb; js += GEMM_R){
	
	jmin = n - j - jb - js;
	if (jmin > GEMM_R) jmin = GEMM_R;

	offsetC = offsetB + js * lda * COMPSIZE;
	
#if GEMM_UNROLL_N > 8
	LASWP_PLUS(jmin, j + offset + 1, j + jb + offset, ZERO, 
#ifdef COMPLEX
		   ZERO,
#endif
		   offsetC - offset * COMPSIZE, lda, NULL, 0 , ipiv, 1);
	
	GEMM_ONCOPY (jb, jmin, offsetC + j * COMPSIZE, lda, sb);
#else
	LASWP_NCOPY(jmin, j + offset + 1, j + jb + offset, 
		    offsetC - offset * COMPSIZE, lda, ipiv, sb);
#endif

	
	for (jc = 0; jc < jb; jc += GEMM_P) {
	  jcmin = jb - jc;
	  if (jcmin > GEMM_P) jcmin = GEMM_P;

	  TRSM_KERNEL_LT(jcmin, jmin, jb, dm1,
#ifdef COMPLEX
			 ZERO,
#endif
			 sa1 + jb * jc * COMPSIZE,
			 sb, offsetC + (j + jc) * COMPSIZE, lda, jc);
	}

	for (is = j + jb; is < m; is += GEMM_P){
	  
	  imin = m - is;
	  if (imin > GEMM_P) imin = GEMM_P;

	  GEMM_ITCOPY (jb, imin, offsetA + is * COMPSIZE, lda, sa2);
	  
	  GEMM_KERNEL_N(imin, jmin, jb, dm1,
#ifdef COMPLEX
			ZERO,
#endif
			sa2, sb, offsetC + is * COMPSIZE, lda);
	}
      }
    }
  }
  
  for (j = 0; j < min_mn; j += blocking) {
    jb = MIN(min_mn - j, blocking);
    LASWP_PLUS(jb, j + jb + offset + 1, min_mn + offset, ZERO,
#ifdef COMPLEX
	       ZERO,
#endif
	       a - (offset - j * lda) * COMPSIZE, lda, NULL, 0 , ipiv, 1);
    
  }

  return info;
}

==> GotoBLAS-1.15.tar.gz: GotoBLAS/lapack/getrf/getrf_single.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include "common.h"

#define GEMM_PQ  MAX(GEMM_P, GEMM_Q)
#define REAL_GEMM_R (GEMM_R - GEMM_PQ)

static FLOAT dm1 = -1.;

blasint CNAME(long m, long  n, FLOAT *a, long lda, blasint *ipiv, long offset, FLOAT *sa, FLOAT *sb){

  long j, js, jmin, is, imin, jc, jcmin;
  blasint iinfo, info;
  long jb, min_mn, blocking;
  FLOAT *offsetA, *offsetB, *offsetC;

  FLOAT *sb2 = (FLOAT *)((((long)sb
		    + GEMM_PQ  * GEMM_Q * COMPSIZE * SIZE + GEMM_ALIGN) & ~GEMM_ALIGN)
		  + GEMM_BUFFER_B_OFFSET);
  
  if (m == 0 || n == 0) return 0;
  
  if (n <= GEMM_UNROLL_N * 2) {
    info = GETF2(m, n, a, lda, ipiv, offset, sb);
    return info;
  }
  
  info = 0;

  blocking = GEMM_Q;
  if (n <= 4 * GEMM_Q) blocking = n / 4;
  
  min_mn = MIN(m, n);

  for (j = 0; j < min_mn; j += blocking) {
    
    jb = min_mn - j;
    if (jb > blocking) jb = blocking;

    offsetA = a +  j       * lda * COMPSIZE;
    offsetB = a + (j + jb) * lda * COMPSIZE;

    iinfo = CNAME(m - j, jb, offsetA + j * COMPSIZE, lda, ipiv, j + offset, sa, sb);

    if (iinfo && !info) info = iinfo + j;

    if (j + jb < n) {
      
      TRSM_ILTCOPY(jb, jb, offsetA + j * COMPSIZE, lda, 0, sb);
      
      for (js = 0; js < n - j - jb; js += REAL_GEMM_R){
	
	jmin = n - j - jb - js;
	if (jmin > REAL_GEMM_R) jmin = REAL_GEMM_R;

	offsetC = offsetB + js * lda * COMPSIZE;
	
#if GEMM_UNROLL_N > 8
	LASWP_PLUS(jmin, j + offset + 1, j + jb + offset, ZERO, 
#ifdef COMPLEX
		   ZERO,
#endif
		   offsetC - offset * COMPSIZE, lda, NULL, 0 , ipiv, 1);
	
	GEMM_ONCOPY (jb, jmin, offsetC + j * COMPSIZE, lda, sb2);
#else
	LASWP_NCOPY(jmin, j + offset + 1, j + jb + offset, 
		    offsetC - offset * COMPSIZE, lda, ipiv, sb2);
#endif

	
	for (jc = 0; jc < jb; jc += GEMM_P) {
	  jcmin = jb - jc;
	  if (jcmin > GEMM_P) jcmin = GEMM_P;

	  TRSM_KERNEL_LT(jcmin, jmin, jb, dm1,
#ifdef COMPLEX
			 ZERO,
#endif
			 sb + jb * jc * COMPSIZE,
			 sb2, offsetC + (j + jc) * COMPSIZE, lda, jc);
	}

	for (is = j + jb; is < m; is += GEMM_P){
	  
	  imin = m - is;
	  if (imin > GEMM_P) imin = GEMM_P;

	  GEMM_ITCOPY (jb, imin, offsetA + is * COMPSIZE, lda, sa);
	  
	  GEMM_KERNEL_N(imin, jmin, jb, dm1,
#ifdef COMPLEX
			ZERO,
#endif
			sa, sb2, offsetC + is * COMPSIZE, lda);
	}
      }
    }
  }
  
  for (j = 0; j < min_mn; j += blocking) {
    jb = MIN(min_mn - j, blocking);
    LASWP_PLUS(jb, j + jb + offset + 1, min_mn + offset, ZERO,
#ifdef COMPLEX
	       ZERO,
#endif
	       a - (offset - j * lda) * COMPSIZE, lda, NULL, 0 , ipiv, 1);
    
  }

  return info;
}
